// @dart=2.9
import 'package:sprintf/sprintf.dart';

class Rectangle {
  ST0 height;
  ST1 length;

  __init__<T0, T1>(T0 height, T1 length) {
    height = height;
    length = length;
  }
}

main() {
  final Rectangle r = Rectangle();
}
