
        //! ```cargo
        //! [package]
        //! edition = "2018"
        //! [dependencies]
        //! 
        //! ```
        

#![allow(non_snake_case)]
#![allow(non_upper_case_globals)]
#![allow(unused_imports)]
#![allow(unused_mut)]
#![allow(unused_parens)]


use std::collections;

pub struct A {
pub B: ST0,
}

impl A {
let B: _ = "FOO"; 
}
pub fn main()  {
assert!(A::B == "FOO");
}

