package main

import (
"fmt")



class Rectangle {
height ST0
length ST1

 func __init__[T0 any, T1 any](self Rectangle, height T0 any, length T1 any) {
self.height = height
self.length = length}


}

func Main() {
var r Rectangle = Rectangle{}}


