class Rectangle {
    val height: ST0
    val length: ST1

    fun __init__ <T0, T1> (height: T0, length: T1) {
        height = height
        length = length
    }
}

fun main() {
    val r = Rectangle()
}
